indexing
   title:       "Date class implementation"
   cluster:     "timedate [timedate]";
   project:     "The Universal Time & Date UTDC";
   copyright:   "SIG Computer, 1995";
   author:      "Alexei Simonov (alex@eiffel.ru)";
   approved_by: "Frieder Monninger";
   original:    "22,Sep,95"
   version:     "1.0"
   done_at:     "SIG Computer (info@eiffel.ru)";
   license:     "GNU Library GPL - see included readme file";
   extrnl_name: "date.e";

class DATE

   inherit
      SIMPLE_TIMESTAMP
         redefine
            infix "<"
         end

   creation
      clear,
      set_date,
      set_from_absolute_days,
      set_to_now

   -------------------------------------------------------------------
   feature -- Commands

      absolute_days : INTEGER;

      set_date( yr, mh, dy : INTEGER ) is
         require
            date_valid : tt.valid_date( yr, mh, dy )
         do
            absolute_days :=
            ( ( yr - tt.one_year ) // tt.leap_year ) * tt.days_4_year +
            ( ( yr - tt.one_year ) \\ tt.leap_year ) * tt.days_in_year -
            ( yr - tt.one_year ) // tt.leap_const1 + ( yr - tt.one_year ) //
            tt.leap_const2;
            if is_leap_year = true and mh > tt.second_month then
               absolute_days := absolute_days + tt.one_day;
            end; -- if

            absolute_days :=
            absolute_days + tt.days_in_months.item( mh ) + dy - tt.one_day;
         end -- set_date

      set_from_absolute_days( i : INTEGER ) is
         require
            absolute_days_valid : i >= tt.zero
         do
            absolute_days := i;
         end -- set_from_absolute_days

      set_to_now is
         local
            now : NOW;
         do
            !!now;
--            set_date( now.year - tt.one_year - tt.one_year,
--                      now.month, now.day );
            set_date( now.year, now.month, now.day );
         end -- set_to_now

      clear is
         do
            absolute_days := tt.zero;
         end -- clear

   -------------------------------------------------------------------
   feature -- Queries

      infix "<" ( ts : like CURRENT ) : BOOLEAN is
         do
            Result := current.absolute_days < ts.absolute_days
         end -- infix "<"

      infix "+",
      plus ( d: LONG_DURATION ) : like CURRENT is
         require else
            date_valid : ( current.absolute_days + d.as_days ) > tt.zero
         local
            sum : INTEGER;
         do
            !!Result.set_from_absolute_days (absolute_days+d.absolute_days);
         end -- infix "+", plus

      infix "-",
      minus ( d: LONG_DURATION ) : like CURRENT is
         require else
            date_valid : ( current.absolute_days - d.whole_days ) > tt.zero
         do
            !!Result.set_from_absolute_days (absolute_days-d.absolute_days);
         end -- infix "-", minus

      difference ( t :  like CURRENT ) :  LONG_DURATION is
         do
            !!Result.set_days ( current.absolute_days - t.absolute_days );
         end -- difference

      day_of_week : INTEGER is
         do
            Result := absolute_days \\ tt.days_in_week + tt.one_day;
         end -- day_of_week

      is_leap_year : BOOLEAN is
         do
            Result := tt.is_leap_year (year_part)
         end -- is_leap_year

      year_part : INTEGER is
         local
            sum, res, yr  : INTEGER;
         do
            sum := absolute_days;
            yr  := sum // tt.days_400_year;
            sum := sum - yr * tt.days_400_year;
            res := yr * tt.leap_const2;
            yr  := sum // tt.days_100_year;
            sum := sum - yr * tt.days_100_year;
            res := res + yr * tt.leap_const1;
            yr  := sum // tt.days_4_year;
            sum := sum - yr * tt.days_4_year;
            res := res + yr * tt.leap_year;
            yr  := sum // tt.days_in_year;

            if yr = tt.leap_year then
               yr := yr - tt.one_year;
            end -- if

            Result := res + yr + tt.one_year;
         end -- year_part

      month_part : INTEGER is
         local
            sum, res, mh  : INTEGER;
         do
            sum :=
            absolute_days - ( ( year_part - tt.one_year ) // tt.leap_year ) *
            tt.days_4_year - ( ( year_part - tt.one_year ) \\ tt.leap_year ) *
            tt.days_in_year + ( year_part - tt.one_year ) // tt.leap_const1 -
            ( year_part - tt.one_year ) // tt.leap_const2 + tt.one_day;
            mh  := tt.zero;
            from
               res := tt.one_month;
            until
               ( tt.days_in_months.item( res ) + mh ) >= sum
            loop
               res := res + tt.one_month;
               if is_leap_year = true and res > tt.second_month then
                  mh := tt.one_day;
               end -- if
            end -- from
            Result := res - tt.one_month;
         end -- month_part

      day_part : INTEGER is
         local
            sum : INTEGER;
         do
            sum :=
            absolute_days - ( ( year_part - tt.one_year ) // tt.leap_year ) *
            tt.days_4_year - ( ( year_part - tt.one_year ) \\ tt.leap_year ) *
            tt.days_in_year + ( year_part - tt.one_year ) // tt.leap_const1 -
            ( year_part - tt.one_year ) // tt.leap_const2 + tt.one_day;
            if is_leap_year = true and month_part > tt.second_month then
               sum := sum - tt.one_day;
            end -- if
            Result := sum - tt.days_in_months.item (month_part);
         end -- day_part

   invariant
      valid_date: tt.valid_date (year_part, month_part, day_part)

end -- class DATE
